package com.composition.android.ad.csj.load.request

import android.content.Context
import com.bytedance.sdk.openadsdk.*
import com.composition.android.ad.csj.impl.CSJNativeAdImageSize
import com.composition.android.lib.ad.basic.AdResult
import com.composition.android.lib.ad.bean.AdUnitBean
import com.composition.android.lib.ad.interfaces.IAdFormatRequest
import com.composition.android.lib.ad.util.printAdFailInfo
import com.composition.android.lib.ad.util.printAdInfo
import kotlinx.coroutines.suspendCancellableCoroutine
import kotlin.coroutines.resume

/**
 * @classname:
 * @author: LiuXiaoLong
 * @date: 2022/9/13
 * description:穿山甲原生模板广告加载类
 */
class CSJNativeTemplateRequestImpl : IAdFormatRequest {

    override suspend fun load(context: Context, adUnitBean: AdUnitBean): AdResult {
        return suspendCancellableCoroutine {

            val imageAcceptedSize = if (adUnitBean.adSize is CSJNativeAdImageSize){
                val adSize = adUnitBean.adSize as CSJNativeAdImageSize
                adSize.width to adSize.height
            }else{
                640 to 320
            }

            val adSlot = AdSlot.Builder()
                .setCodeId(adUnitBean.adUnitId)
                .setAdCount(1)
                .setImageAcceptedSize(imageAcceptedSize.first, imageAcceptedSize.second)
                .setAdLoadType(TTAdLoadType.PRELOAD)
                .build()

            TTAdSdk.getAdManager().createAdNative(context)
                .loadNativeExpressAd(adSlot, object : TTAdNative.NativeExpressAdListener {
                    override fun onError(p0: Int, p1: String?) {
                        printAdFailInfo(p0, p1)
                        if (it.isActive) {
                            it.resume(AdResult.Fail(adUnitBean, msg = p1 ?: "穿山甲广告-原生模板-加载失败"))
                        }
                    }

                    override fun onNativeExpressAdLoad(p0: MutableList<TTNativeExpressAd>?) {
                        printAdInfo(adUnitBean)
                        if (it.isActive) {
                            if (p0.isNullOrEmpty().not()) {
                                it.resume(AdResult.Success(adBean = adUnitBean, adObject = p0?.get(0), msg = "穿山甲广告-原生模板-加载成功"))
                            } else {
                                it.resume(AdResult.Fail(adUnitBean, msg = "穿山甲广告原生模板-加载失败, SDK返回的广告实例为空"))
                            }
                        }
                    }
                })
        }
    }
}